/******************************************************************************
 *
 *                   INTEL CORPORATION PROPRIETARY INFORMATION
 *       This software is supplied under the terms of a license agreement or
 *       nondisclosure agreement with Intel Corporation and may not be copied
 *       or disclosed except in accordance with the terms of that agreement.
 *
 *            Copyright (c) 2008-2009 Intel Corporation. All Rights Reserved.
 *
 *       All rights reserved.  No part of this program or publication may be
 *       reproduced, transmitted, transcribed, stored in a retrieval system,
 *       or translated into any language or computer language, in any form or
 *       by any means, electronic, mechanical, magnetic, optical, chemical,
 *       manual, or otherwise, without the prior written permission of Intel
 *       Corporation.
 *
 *
 *******************************************************************************/
#ifndef __CR_OSAL_H_
#define __CR_OSAL_H_

#ifdef CR_LINUX
#include <os/include/linux/cr_osal_linux.h>
#else

#ifdef CR_WINDOWS
#include <os/include/windows/cr_osal_windows.h>
#else

#ifdef CR_EFI
#include <os/include/efi/cr_osal_efi.h>
#endif //ifdef CR_EFI

#endif //ifdef CR_WINDOWS

#endif //ifdef CR_LINUX

#include <os/include/cr_exception_class.h>

//Namespaces
using namespace std;
//using namespace boost;
using namespace Module;
using namespace Osal;

//Common Typedefs
#ifdef CR_LINUX
typedef unsigned char 	    UINT8;
#endif
typedef unsigned short      UINT16;
typedef unsigned long long  ULONGLONG;
typedef char                CHAR;
typedef short               INT16;
typedef int 	            INTN;

TYPEDEF_MAPOF(int, string, ErrorStringMap);

typedef long long           LONGLONG;

typedef wstring				UNICODE_STR;
//typedef shared_ptr<UINT8>	BYTE_PTR;
//typedef shared_ptr<void>	VOID_PTR;

typedef enum _NODE_TYPE
{
	NODE_LEAF = 0,
	NODE_NONLEAF
} NODE_TYPE;

struct _XNode;
typedef struct _XNode XNode;

typedef STRING XToken;

TYPEDEF_VECTOROF(XToken, XTokenList);

TYPEDEF_MAPOF(STRING, XNode, ResultMap);

typedef STRING_TO_STRING_MAP XSettingMap;

typedef map<string,STRING_TO_STRING_MAP> SettingsMap;

struct _XNode {

	NODE_TYPE mType;
	XTokenList mXNodeList;
	XSettingMap mOptionMap;
	_XNode()
		: mXNodeList()
	{}
	~_XNode()
	{
		mXNodeList.clear();
	}
};


typedef STRING_TO_STRING_HMAP   STRING_STRING_MAP;

TYPEDEF_HASH_MAPOF(STRING, VECTOROF_STRING, STRING_VECTOR_STRING_MAP);

TYPEDEF_HASH_MAPOF(STRING, STRING_STRING_MAP, STRING_STRING_STRING_MAP);

TYPEDEF_HASH_MAPOF(STRING, STRING_VECTOR_STRING_MAP, STRING_STRING_VECTOR_STRING_MAP);

TYPEDEF_MAPOF(int, string, INT_TO_STRING_MAP);
TYPEDEF_MAPOF(int, VECTOROF_STRING, INT_TO_VECTOR_STRING_MAP);
TYPEDEF_MAPOF(string, VECTOROF_STRING, STRING_TO_VECTOR_STRING_MAP);

#define _GET "GET:/"
#define _SET "SET:/"
#define UNSUPPORTED      "UNSUPPORTED"

typedef enum _FN_NAME {
	FN_IPMI,
	FN_IPMI_RESPONSE,
	FN_LOOKUP_TABLE,
	FN_STORE,
	FN_COPY,
	FN_APPEND,
	FN_TEMPLATE,
	FN_SET,
	FN_MAP,
	FN_MASK,
	FN_IF,
	FN_FOR,
	FN_WHILE,
	FN_EXPRESSION,
	FN_FORMAT_STRING,
	FN_SPLIT,
	FN_JOIN, 
	FN_PAD, 
	FN_CONVERT,
	FN_END,
	FN_FLATTEN,
	FN_FILTER,
	FN_CREATE_FILTER,
	FN_READ,
	FN_PRINT,
	FN_EXCLUDE,
	FN_SIZE,
	FN_RETURN,
	FN_DEBUG,
	FN_ASSERT,
	FN_THROW,
	FN_ELSEIF,
	FN_ELSE,
	FN_SLEEP,
	FN_INCREMENT,
	FN_STRING_LENGTH,
	FN_ASSIGN,
	FN_PREPEND,
	FN_POLL,
    FN_IPMI_WITHOUT_THROW,
	FN_MATHEMATICAL,
	FN_TEMPLATE_WITH_CATCH
} FN_NAME;

enum TYPE { STRING_TYPE, VECTOR_STRING_TYPE };

#define IPMI_MAX_BUFFER  255
#define DECIMAL_RANGE    "0123456789"
#define _OP_EQUAL_TO     "=="
#define _OP_NOT_EQUAL_TO "!="
#define _OP_LESS_THAN    "<"
#define _OP_GREATER_THAN ">"
#define _OP_GREATER_THAN_OR_EQUAL_TO ">="
#define _OP_LESS_THAN_OR_EQUAL_TO "<="

#define STR_SIZE_CHECK          "STR_SIZE_CHECK"
#define RANGE_CHECK          "RANGE_CHECK"
#define HEX2DEC          "HEX2DEC"
#define DEC2HEX          "DEC2HEX"
#define HEX2STR          "HEX2STR"
#define STR2HEX          "STR2HEX"
#define BIT_OR           "BIT_OR"
#define MASK             "MASK"
#define LEFT_SHIFT       "LEFT_SHIFT"
#define RIGHT_SHIFT      "RIGHT_SHIFT"

//enum OPERATION { READ, WRITE, DELETE_OP };

#define localST(x) mST[x]
#define sT(x) Module::BMCConfig::SymbolTable::instance()[x]
#define sTGet(x) Module::BMCConfig::SymbolTable::instance().get(x)
//#define sTGet((x)) Module::BMCConfig::SymbolTable::instance().get(x)

#define fD(x) Module::BMCConfig::IPMIFilterDB::instance()[x]
#define fDGet(x) Module::BMCConfig::IPMIFilterDB::instance().get(x)

#define tB(x) Module::BMCConfig::TableDB::instance()[x]
#define tBGet(x) Module::BMCConfig::TableDB::instance().get(x)

#define pD(x) Module::BMCConfig::IPMIPacketDB::instance()[x]
#define pDGet(x) Module::BMCConfig::IPMIPacketDB::instance().get(x)

#define tD(x) Module::BMCConfig::TemplateDB::instance()[x]
#define tDGet(x) Module::BMCConfig::TemplateDB::instance().get(x)

#define gB(x) Module::BMCConfig::SymbolTable::instance().mark_as_garbage(x)
#define sGB(x) Module::BMCConfig::SymbolTable::instance().mark_as_stinking_garbage(x)

#define gBC() Module::BMCConfig::SymbolTable::instance().garbage_collector()
#define sGBC() Module::BMCConfig::SymbolTable::instance().stinking_garbage_collector()

typedef struct _MEMINFO{
	string memload;
	string totalram;
	string freeram;
	string totalswap;
	string freeswap;
	string totalvirt;
	string freevirt;
} MEMINFO;

typedef struct _OSINFO{
	string hostname;
	string domain;
	string version;
	string machine;
	string type;
	string build;

}OSINFO;

typedef struct _DRIVEINFO{
	string Id;
	string TotalSpace;
	string FreeSpace;
}DRIVEINFO;

typedef struct _NICINFO{
	string Name;
	string TransmitPackets;
	string TransmitBytes;
	string TransmitErrors;
	string ReceivePackets;
	string ReceiveBytes;
	string ReceiveErrors;
}NICINFO;
//end

void GetMemInfo(MEMINFO&);
void GetOSInfo(OSINFO&);
bool GetOSDisplayString(string& os_display_string);
void GetDriveData(vector<DRIVEINFO>&);
void GetNICInfo(vector<NICINFO>&);

#endif //ifndef __CR_OSAL_H_
